package javax.swing.text;

import java.io.Serializable;

abstract class GapVector implements Serializable
{
    private Object array;
    private int g0;
    private int g1;
    
    public GapVector() {
        this(10);
    }
    
    public GapVector(int g1) {
        this.array = this.allocateArray(g1);
        this.g0 = 0;
        this.g1 = g1;
    }
    
    protected abstract Object allocateArray(int p0);
    
    protected abstract int getArrayLength();
    
    protected final Object getArray() {
        return this.array;
    }
    
    protected final int getGapStart() {
        return this.g0;
    }
    
    protected final int getGapEnd() {
        return this.g1;
    }
    
    protected void replace(int n, int n2, Object o, int n3) {
        int n4 = 0;
        if (n3 == 0) {
            this.close(n, n2);
            return;
        }
        if (n2 > n3) {
            this.close(n + n3, n2 - n3);
        }
        else {
            int n5 = n3 - n2;
            System.arraycopy(o, n2, this.array, this.open(n + n2, n5), n5);
            n3 = n2;
        }
        System.arraycopy(o, n4, this.array, n, n3);
        return;
    }
    
    void close(int n, int n2) {
        if (n2 == 0) {
            return;
        }
        int n3 = n + n2;
        int n4 = this.g1 - this.g0 + n2;
        if (n3 <= this.g0) {
            if (this.g0 != n3) {
                this.shiftGap(n3);
            }
            this.shiftGapStartDown(this.g0 - n2);
        }
        else if (n >= this.g0) {
            if (this.g0 != n) {
                this.shiftGap(n);
            }
            this.shiftGapEndUp(this.g0 + n4);
        }
        else {
            this.shiftGapStartDown(n);
            this.shiftGapEndUp(this.g0 + n4);
        }
        return;
    }
    
    int open(int n, int n2) {
        int n3 = this.g1 - this.g0;
        if (n2 == 0) {
            if (n > this.g0) {
                n += n3;
            }
            return n;
        }
        else {
            this.shiftGap(n);
            if (n2 >= n3) {
                this.shiftEnd(this.getArrayLength() - n3 + n2);
                int n4 = this.g1 - this.g0;
            }
            this.g0 += n2;
            return n;
        }
    }
    
    void resize(int a) {
        Object allocateArray = this.allocateArray(a);
        System.arraycopy(this.array, 0, allocateArray, 0, Math.min(a, this.getArrayLength()));
        this.array = allocateArray;
    }
    
    protected void shiftEnd(int n) {
        int arrayLength = this.getArrayLength();
        int g1 = this.g1;
        int n2 = arrayLength - g1;
        int newArraySize = this.getNewArraySize(n);
        int g2 = newArraySize - n2;
        this.resize(newArraySize);
        this.g1 = g2;
        if (n2 != 0) {
            System.arraycopy(this.array, g1, this.array, g2, n2);
        }
    }
    
    int getNewArraySize(int n) {
        return (n + 1) * 2;
    }
    
    protected void shiftGap(int g0) {
        if (g0 == this.g0) {
            return;
        }
        int g2 = this.g0;
        int n = g0 - g2;
        int g3 = this.g1;
        int g4 = g3 + n;
        this.g0 = g0;
        this.g1 = g4;
        if (n > 0) {
            System.arraycopy(this.array, g3, this.array, g2, n);
        }
        else if (n < 0) {
            System.arraycopy(this.array, g0, this.array, g4, -n);
        }
        return;
    }
    
    protected void shiftGapStartDown(int g0) {
        this.g0 = g0;
    }
    
    protected void shiftGapEndUp(int g1) {
        this.g1 = g1;
    }
}
